package com.ningdatech.pmapi.meeting.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ningdatech.pmapi.meeting.entity.domain.MeetingExpert;
import com.ningdatech.pmapi.meeting.entity.dto.MeetingAndAttendStatusDTO;
import com.ningdatech.pmapi.meeting.entity.dto.ReviewProjectDTO;
import com.ningdatech.pmapi.meeting.entity.enumeration.ExpertAttendStatusEnum;
import com.ningdatech.pmapi.meeting.entity.req.ReviewProjectListReq;
import org.apache.ibatis.annotations.Param;

import java.util.Collection;
import java.util.List;

/**
 * <p>
 * 事务专家表 Mapper 接口
 * </p>
 *
 * @author WendyYang
 * @since 2022-07-27
 */
public interface MeetingExpertMapper extends BaseMapper<MeetingExpert> {

    /**
     * 根据专家状态查询需要参加的会议信息
     *
     * @param expertId   专家ID
     * @param status     状态
     * @param meetingIds 会议ID
     * @return 专家参会状态列表
     * @author WendyYang
     **/
    List<MeetingAndAttendStatusDTO> selectByExpertIdAndStatus(@Param("expertId") Long expertId,
                                                              @Param("status") Integer status,
                                                              @Param("meetingIds") List<Long> meetingIds);

    /**
     * 分页查询专家列表
     *
     * @param page       分页数据
     * @param status     状态{@link ExpertAttendStatusEnum}
     * @param meetingId  会议ID
     * @param inviteType 邀请类型
     * @return Page<MeetingExpert>
     * @author WendyYang
     **/
    Page<MeetingExpert> selectExpertByStatusAndMeetingId(Page<MeetingExpert> page, @Param("status") Integer status,
                                                         @Param("meetingId") Long meetingId,
                                                         @Param("inviteType") Integer inviteType);

    /**
     * 分页查询专家列表
     *
     * @param page       分页数据
     * @param status     状态{@link ExpertAttendStatusEnum}
     * @param meetingIds 会议ID
     * @return Page<MeetingExpert>
     * @author WendyYang
     **/
    Page<MeetingExpert> selectExpertByStatusAndMeetingIds(Page<MeetingExpert> page, @Param("status") Integer status,
                                                          @Param("meetingIds") Collection<Long> meetingIds);

    /**
     * 根据会议ID与参与状态统计专家数量
     *
     * @param status     会议状态
     * @param meetingId  会议ID
     * @param inviteType 邀请类型
     * @return 数量
     */
    int countExpertByStatusAndMeetingId(@Param("status") Integer status, @Param("meetingId") Long meetingId, @Param("inviteType") Integer inviteType);

    /**
     * 查询会议的所有被抽取人最后一条记录
     *
     * @param meetingIds 会议ID
     * @return 抽取记录
     * @author WendyYang
     **/
    List<MeetingExpert> listExpertLastByMeetingIds(@Param("meetingIds") Collection<Long> meetingIds);

    /**
     * 查询专家待评审项目
     *
     * @param page 分页
     * @param req  查询参数
     * @return 待评审项目分页数据
     * @author WendyYang
     **/
    Page<ReviewProjectDTO> pageReviewProjectList(Page<ReviewProjectDTO> page, @Param("p") ReviewProjectListReq req);

}
