package com.ningdatech.pmapi.projectlib.handle;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import com.ningdatech.basic.util.NdDateUtils;
import com.ningdatech.pmapi.common.constant.CommonConst;
import com.ningdatech.pmapi.projectlib.enumeration.InstTypeEnum;
import com.ningdatech.pmapi.projectlib.model.entity.ProjectInst;
import com.ningdatech.pmapi.projectlib.service.IProjectInstService;
import com.ningdatech.pmapi.todocenter.utils.BuildUserUtils;
import com.wflow.workflow.bean.process.ProgressNode;
import com.wflow.workflow.bean.vo.ProcessDetailVO;
import com.wflow.workflow.bean.vo.ProcessProgressVo;
import com.wflow.workflow.enums.ProcessStatusEnum;
import com.wflow.workflow.enums.StepStatusEnum;
import com.wflow.workflow.service.ProcessInstanceService;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;

/**
 * 项目预审处理
 *
 * @author CMM
 * @since 2023/02/27 10:09
 */
@Order(3)
@Component
public class PreliminaryPreviewHandle extends AbstractProcessBusinessHandle {

    private final IProjectInstService projectInstService;
    private final ProcessInstanceService processInstanceService;
    private final BuildUserUtils buildUserUtils;

    public PreliminaryPreviewHandle(IProjectInstService projectInstService, ProcessInstanceService processInstanceService, BuildUserUtils buildUserUtils) {
        this.projectInstService = projectInstService;
        this.processInstanceService = processInstanceService;
        this.buildUserUtils = buildUserUtils;
    }

    @Override
    void businessHandle(Long projectId, List<ProcessDetailVO> processSchedule) {
        ProcessDetailVO processDetailVO = new ProcessDetailVO();
        // 根据项目ID查询项目预审流程的流程状态
        ProjectInst projectInst = projectInstService.getOne(Wrappers.lambdaQuery(ProjectInst.class)
                .eq(ProjectInst::getProjectId, projectId)
                .eq(ProjectInst::getInstType, InstTypeEnum.DEPT_UNITED_REVIEW.getCode())
                .orderByDesc(ProjectInst::getCreatOn)
                .last("limit 1"));
        if (Objects.isNull(projectInst)){
            processDetailVO.setStepStatus(StepStatusEnum.NOT_START);
            processDetailVO.setProcessName(CommonConst.PRELIMINARY_PREVIEW);
            processSchedule.add(processDetailVO);
            return;
        }
        String instCode = projectInst.getInstCode();
        ProcessProgressVo instanceDetail = processInstanceService.getProgressInstanceDetail(null, instCode);
        String status = instanceDetail.getStatus();
        if (ProcessStatusEnum.UNDER_REVIEW.getDesc().equals(status)){
            processDetailVO.setStepStatus(StepStatusEnum.ON_GOING);
        } else if (ProcessStatusEnum.BE_REJECTED.getDesc().equals(status)) {
            processDetailVO.setStepStatus(StepStatusEnum.REJECTED);
        } else if (ProcessStatusEnum.APPROVED.getDesc().equals(status)) {
            processDetailVO.setStepStatus(StepStatusEnum.COMPLETED);
        }

        // 装配节点审核人员信息
        List<ProgressNode> progressInfo = instanceDetail.getProgressInfo();
        buildUserUtils.buildUserByProcessInfo(progressInfo);

        // 根据流程处理详情获取流程完成时间
        if(StepStatusEnum.contains(processDetailVO.getStepStatus(),
                Lists.newArrayList(StepStatusEnum.REJECTED,
                        StepStatusEnum.COMPLETED))){
            ProgressNode progressNode = progressInfo.get(progressInfo.size() - 1);
            LocalDateTime finishTime = NdDateUtils.date2LocalDateTime(progressNode.getFinishTime());
            processDetailVO.setFinishTime(finishTime);
        }
        processDetailVO.setProcessProgressVo(instanceDetail);
        processDetailVO.setProcessName(CommonConst.PRELIMINARY_PREVIEW);
        processSchedule.add(processDetailVO);
    }
}
