package com.ningdatech.pmapi.todocenter.utils;

import cn.hutool.core.collection.CollUtil;
import com.google.common.collect.Sets;
import com.ningdatech.pmapi.user.service.IUserInfoService;
import com.wflow.workflow.bean.dto.ProcessInstanceUserDto;
import com.wflow.workflow.bean.process.ProgressNode;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 装配节点用户信息工具类
 *
 * @author CMM
 * @since 2023/03/08 11:52
 */
@Slf4j
@RequiredArgsConstructor
@Component
public class BuildUserUtils {

    private final IUserInfoService userInfoService;

    public void buildUserByProcessInfo(List<ProgressNode> progressInfo) {
        Set<String> userSet = Sets.newHashSet();
        progressInfo.forEach(node -> {
            if (CollUtil.isNotEmpty(node.getChildren())) {
                for (ProgressNode innerNode : node.getChildren()) {
                    userSet.add(innerNode.getUserId());
                }
            } else {
                userSet.add(node.getUserId());
            }
        });
        Map<String, ProcessInstanceUserDto> userMap = userInfoService.getUserMapByIds(userSet);
        buildUser(progressInfo,userMap);
    }
    private void buildUser(List<ProgressNode> progressInfo, Map<String, ProcessInstanceUserDto> userMap) {
        for (ProgressNode progressNode : progressInfo) {
            progressNode.setUser(userMap.get(progressNode.getUserId()));
            if (CollUtil.isNotEmpty(progressNode.getChildren())) {
                buildUser(progressNode.getChildren(), userMap);
            }
        }
    }


}
